@@ -13,7 +13,7 @@ use AutoLoader qw(AUTOLOAD);
 
 @ISA = qw(Exporter DynaLoader);
 
-$VERSION = do { my @r = (q$Revision: 0.23 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+$VERSION = do { my @r = (q$Revision: 0.24 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
 
 @EXPORT_OK = qw(
 	get1char
@@ -24,6 +24,8 @@ $VERSION = do { my @r = (q$Revision: 0.23 $ =~ /\d+/g); sprintf "%d."."%02d" x $
 	put32
 	getIPv4
 	putIPv4
+	getIPv6
+	putIPv6
 	getstring
 	putstring
 	dn_comp
@@ -42,6 +44,9 @@ $VERSION = do { my @r = (q$Revision: 0.23 $ =~ /\d+/g); sprintf "%d."."%02d" x $
 	put_arcount
 	inet_aton
 	inet_ntoa
+	ipv6_aton
+	ipv6_n2x
+	ipv6_n2d
 	sec2time
 	ttlAlpha2Num
 	collapse
@@ -118,6 +123,8 @@ Net::DNS::ToolKit - tools for working with DNS packets
 	put32
 	getIPv4
 	putIPv4
+	putIPv6
+	getIPv6
 	getstring
 	putstring
 	dn_comp
@@ -136,6 +143,9 @@ Net::DNS::ToolKit - tools for working with DNS packets
 	put_arcount
 	inet_aton
 	inet_ntoa
+	ipv6_aton
+	ipv6_n2x
+	ipv6_n2d   
 	sec2time
 	ttlAlpha2Num
 	collapse
@@ -161,6 +171,8 @@ Net::DNS::ToolKit - tools for working with DNS packets
   $newoff = put_arcount(\$buffer,$int);
   ($netaddr,$newoff)=getIPv4(\$buffer,$offset);
   $newoff = putIPv4(\$buffer,$offset,$netaddr);
+  ($ipv6addr,$newoff)=getIPv6(\$buffer,$offset);
+  $newoff = putIPv6(\$buffer,$offset,$ipv6addr);
   ($offset,
    $id,$qr,$opcode,$aa,$tc,$rd,$ra,$mbz,$ad,$cd,$rcode,
    $qdcount,$ancount,$nscount,$arcount)
@@ -172,6 +184,9 @@ Net::DNS::ToolKit - tools for working with DNS packets
   ($newoff,@dnptrs)=dn_comp(\$buffer,$offset,\$name,\@dnptrs);
   $dotquad = inet_ntoa($netaddr);
   $netaddr = inet_aton($dotquad);
+  $ipv6addr = ipv6_aton($ipv6_text);
+  $hex_text = ipv6_n2x($ipv6addr);
+  $dec_text = ipv6_n2d($ipv6addr);
   $timetxt = sec2time($seconds);
   $seconds = ttlAlpha2Num($timetext);
   $shorthost = collapse($zonename,$longhost);
@@ -194,6 +209,7 @@ scalar context.
   ($int,$newoff)	= get16(...
   ($long,$newoff)	= get32(...
   ($netaddr,$newoff)	= getIPv4(...
+  ($ipv6addr,$newoff)	= getIPv6(...
   ($string,$newoff)	= getstring(...
   ($newoff,$name)	= dn_expand(...
   ($secs,$usecs)	= gettimeofday(...
@@ -209,6 +225,7 @@ These functions return only a value or an offset.
   $newoff	= put_nscount(...
   $newoff	= put_arcount(...
   $newoff	= putIPv4(...
+  $newoff	= putIPv4(...
   $newoff	= putstring(...
   $newoff	= newhead(...
   $flags	= getflags(...
@@ -456,14 +473,38 @@ In SCALAR context, returns just netaddr.
 
 =item * $newoff = putIPv4(\$buffer,$offset,$netaddr);
 
-Put a netaddr into the buffer. Return the value of
-the new offset pointer to the the next char (usually end of buffer).
+Put a netaddr into the buffer. Return the value of the
+new offset pointer to the next char (usually end of buffer).
 
   input:	pointer to buffer,
 		offset into buffer,
 		packed IPv4 net address
   returns:	pointer to end of buffer
 
+=item * ($ipv6addr,$newoff)=getIPv6(\$buffer,$offset);
+
+Get an IPv6 newtork address from the buffer at $offset. Return the
+ipv6addr and a new offset pointing at the next character beyond.
+
+Returns and empty array on error.
+
+  input:	pointer to buffer,
+		offset into buffer
+  returns:	ipv6addr,
+		offset + size of ipv6addr
+
+IN SCALAR context, returns just ipv6addr.
+
+=item * $newoff = putIPv6(\$buffer,$offset,$ipv6addr);
+
+Put an ipv6addr into the buffer. Return the value of the
+new offset pointer to the next char (usually end of buffer).
+
+  input:	pointer to buffer,
+		offset into buffer,
+		128 bit IPv6 net address
+  returns:	pointer to end of buffer
+
 =item * ($string,$newoff) =
 	getstring(\$buffer,$offset,$length);
 
@@ -679,6 +720,82 @@ Convert a dot-quad IP address into an IPv4 packed network address.
   input:	IP address i.e. 192.5.16.32
   returns:	packed network address
 
+=item * $ipv6addr = ipv6_aton($ipv6_text);
+
+Takes an IPv6 address of the form described in rfc1884
+and returns a 128 bit binary RDATA string.
+
+  input:	ipv6 text
+  returns:	128 bit RDATA string
+
+=cut
+
+sub ipv6_aton {
+  my($ipv6) = @_;
+  return undef unless $ipv6;
+  if ($ipv6 =~ /:(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/) {	# mixed hex, dot-quad
+    return undef if $1 > 255 || $2 > 255 || $3 > 255 || $4 > 255;
+    $ipv6 = sprintf("%s:%X%02X:%X%02X",$`,$1,$2,$3,$4);			# convert to pure hex
+  }
+  return undef unless ($ipv6 =~ /(::|:[a-zA-Z0-9]+)$/);			# pure hex
+  my $c = $ipv6 =~ tr/:/:/;						# count the colons
+  return undef if $c < 7 && $ipv6 !~ /::/;
+  while ($c++ < 7) {							# expand compressed fields
+    $ipv6 =~ s/::/:::/;
+  }
+#  my @hex = split(/:/,$ipv6);						# FAILS on :::::::
+  $ipv6 =~ /(\w*):(\w*):(\w*):(\w*):(\w*):(\w*):(\w*):(\w*)/;		# 8, 16 bit pieces
+  my @hex = ($1 || 0,$2 || 0,$3 || 0,$4 || 0,$5 || 0,$6 || 0,$7 || 0,$8 || 0);
+  $ipv6 = '';
+  my $off = 0;
+  foreach(@hex) {
+    $off = put16(\$ipv6,$off,hex($_));
+  }
+  return $ipv6;
+}
+
+sub _ipv6n2ary {
+  my($buf) = @_;
+  my @hex;
+  foreach(0..7) {
+    ($hex[$_]) = get16(\$buf,$_ << 1);
+  }
+  return @hex;
+}
+
+=item * $hex_text = ipv6_n2x($ipv6addr);
+
+Takes an IPv6 RDATA string and returns an 8 segment IPv6 hex address
+
+  input:	128 bit RDATA string
+  returns:	x:x:x:x:x:x:x:x
+
+=cut
+
+sub ipv6_n2x {
+  return sprintf("%X:%X:%X:%X:%X:%X:%X:%X",&_ipv6n2ary(@_));
+}
+
+=item * $dec_text = ipv6_n2d($ipv6addr);
+
+Takes an IPv6 RDATA string and returns a mixed hex - decimal IPv6 address
+with the 6 uppermost chunks in hex and the lower 32 bits in dot-quad
+representation.
+
+  input:	128 bit RDATA string
+  returns:	x:x:x:x:x:x:d.d.d.d
+
+=cut
+
+sub ipv6_n2d {
+  my @hex = (_ipv6n2ary($_[0]),0,0);
+  $hex[9] = $hex[7] & 0xff;
+  $hex[8] = $hex[7] >> 8;
+  $hex[7] = $hex[6] & 0xff;
+  $hex[6] >>= 8;
+  return sprintf("%X:%X:%X:%X:%X:%X:%d.%d.%d.%d",@hex);
+}
+
 =item * $timetxt = sec2time($seconds);
 
 Convert numeric seconds into a string of the form
@@ -889,6 +1006,8 @@ put16
 put32
 getIPv4
 putIPv4
+getIPv6
+putIPv6
 getstring
 putstring
 dn_comp
@@ -907,6 +1026,9 @@ put_nscount
 put_arcount
 inet_aton
 inet_ntoa
+ipv6_aton
+ipv6_n2x
+ipv6_n2d   
 sec2time
 ttlAlpha2Num
 collapse
